/*
 * decoder.h
 *
 * Description of this file:
 *    Decoder functions definition of the xrddec library
 *
 * --------------------------------------------------------------------------
 *
 *    xrddec - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */

#ifndef __AVS2_DECODER_H__
#define __AVS2_DECODER_H__
#ifdef __cplusplus
extern "C" {
#endif

#include "common.h"

xrddec_t *decoder_open(xrddec_mgr_t *mgr, xrddec_t *h);
void *decoder_decode_picture_data(void *arg1, int arg2);
#if XRDDEC_API_VERSION < 2
void decoder_flush(xrddec_mgr_t *mgr);
#endif
void decoder_close(xrddec_t *h);
int  create_freepictures(xrddec_mgr_t *mgr, int w, int h, int size);
void destroy_freepictures(xrddec_mgr_t *mgr);
int  decoder_alloc_extra_buffer(xrddec_t *h);
void decoder_free_extra_buffer(xrddec_t *h);
void xrddec_write_a_frame(xrddec_picture_t *pic, xrddec_frame_t *frame);

int  task_get_references(xrddec_t *h, int64_t pts, int64_t dts);

void task_unload_packet(xrddec_t *h, es_unit_t *es_unit);
#if XRDDEC_API_VERSION >= 2
int decoder_get_output(xrddec_mgr_t *mgr, xrddec_seq_info_t *headerset, xrddec_picture_t *out_frame, int is_flush);
#endif

#ifdef __cplusplus
}
#endif
#endif  // __AVS2_DECODER_H__
