/*
 * primitives.cc
 *
 * Description of this file:
 *    function handles initialize functions definition of the xrddec library
 *
 * --------------------------------------------------------------------------
 *
 *    xrddec - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */


#include "common.h"
#include "primitives.h"
#include "cpu.h"
#include "intra.h"
#include "mc.h"
#include "transform.h"
#include "quant.h"
#include "deblock.h"
#include "sao.h"
#include "alf.h"

/* ---------------------------------------------------------------------------
 */
ao_funcs_t gf_xrddec;

/* ---------------------------------------------------------------------------
 */
void init_all_primitives(uint32_t cpuid)
{
    memset(&gf_xrddec, 0, sizeof(gf_xrddec));
    gf_xrddec.cpuid = cpuid;

    /* init function handles */
    xrddec_memory_init    (cpuid, &gf_xrddec);
    xrddec_intra_pred_init(cpuid, &gf_xrddec);
    xrddec_pixel_init     (cpuid, &gf_xrddec);
    xrddec_mc_init        (cpuid, &gf_xrddec);
    xrddec_quant_init     (cpuid, &gf_xrddec);
    xrddec_dct_init       (cpuid, &gf_xrddec);
    xrddec_deblock_init   (cpuid, &gf_xrddec);
    xrddec_sao_init       (cpuid, &gf_xrddec);
    xrddec_alf_init       (cpuid, &gf_xrddec);

}
