/*
 * threadpool.h
 *
 * Description of this file:
 *    thread pooling functions definition of the xrddec library
 *
 * --------------------------------------------------------------------------
 *
 *    xrddec - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */

#ifndef __XRDDEC_THREADPOOL_H__
#define __XRDDEC_THREADPOOL_H__
#ifdef __cplusplus
extern "C" {
#endif

typedef struct xrddec_threadpool_t xrddec_threadpool_t;
typedef void *(*xrddec_threadpool_func_t)(void *arg1, int arg2);

int   xrddec_threadpool_init  (xrddec_threadpool_t **p_pool, int threads,
                              xrddec_threadpool_func_t init_func, void *init_arg1, int init_arg2);
void  xrddec_threadpool_run   (xrddec_threadpool_t *pool, xrddec_threadpool_func_t func, void *arg1, int arg2, int wait_sign);
int   xrddec_threadpool_is_free(xrddec_threadpool_t *pool);
void *xrddec_threadpool_wait  (xrddec_threadpool_t *pool, void *arg1, int arg2);
void  xrddec_threadpool_delete(xrddec_threadpool_t *pool);

#ifdef __cplusplus
}
#endif
#endif  // __STARAVS_THREADPOOL_H__
