/*
 * presets.c
 *
 * Description of this file:
 *    parse preset level functions definition of the xRD library
 *
 * --------------------------------------------------------------------------
 *
 *    xRD - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */

#include "common/common.h"
#include "encoder/aec.h"
#include "presets.h"

/**
 * ===========================================================================
 * local tables
 * ===========================================================================
 */
/* ---------------------------------------------------------------------------
 * ֡ȿRDOģʽӦͬpreset
 */
static const uint8_t INTRA_FULL_RDO_NUM[MAX_CU_SIZE_IN_BIT + 1] = {
    0, 0, 9, 9, 9, 9, 9        /* 9:  1x1, 2x2, 4x4, 8x8, 16x16, 32x32, 64x64 */
};

/**
 * ---------------------------------------------------------------------------
 * Function   : modify configurations.
 * Parameters :
 *   [in/out] : p_param        - the coding parameter to be set
 * Return     : none
 * ---------------------------------------------------------------------------
 */
void parse_preset_level(xrd_param_t *p_param)
{
    if (p_param->successive_Bframe != 0) {
        p_param->enable_tdrdo = p_param->enable_tdrdo;
    } else {
        p_param->enable_tdrdo = 1;
    }

    p_param->num_max_ref = 4;
    p_param->lcu_bit_level = 6;

    /* --------------------------- Ԥ ---------------------------
    */
    p_param->inter_2pu = 1;
    p_param->enable_intra = 1;
    p_param->enable_f_frame = 1;
    p_param->enable_mhp_skip = 1;
    p_param->enable_wsm = 1;
    p_param->enable_dhp = 1;
    p_param->enable_dmh = 1;
    p_param->enable_amp = 1;  // NSQT

    /* --------------------------- 任 ---------------------------
    * NSQTSDIPһͬʱ */
    p_param->enable_sdip = 1;
    p_param->enable_nsqt = 1;
    p_param->enable_secT = 1;    // е¾

    /* ---------------------------  ---------------------------
    */
    p_param->i_rdoq_level = RDOQ_ALL;

    /* --------------------------- RDO ---------------------------
    */
    p_param->i_rd_level = RDO_ALL;

    /* --------------------------- ر ---------------------------
    */
    p_param->rdo_bit_est_method = 0;

    /* --------------------------- ˲ ---------------------------
    */
    p_param->enable_alf = 1;
    p_param->enable_sao = 1;

    /* ---------------------------  ---------------------------
    */
    p_param->enable_hadamard = 1;
    p_param->enable_tdrdo = p_param->enable_tdrdo;

    /* tell the encoder preset configuration is utilized */
    p_param->is_preset_configured = TRUE;
}

/* ---------------------------------------------------------------------------
 * reconfigure encoder during encoding
 */
void xrd_reconfigure_encoder(xrd_t *h)
{
    UNUSED_PARAMETER(h);
}

/**
 * ---------------------------------------------------------------------------
 * Function   : set fast algorithms enabled according to different preset levels
 * Parameters :
 *      [in ] : h - pointer to struct xrd_t, the XRDENC encoder
 * Return     : none
 * ---------------------------------------------------------------------------
 */
void encoder_set_fast_algorithms(xrd_t *h)
{

    memcpy(h->tab_num_intra_rdo, INTRA_FULL_RDO_NUM, sizeof(h->tab_num_intra_rdo));

    /* RMD㷨Ƕ */
    /* ---------------------------------------------------------------------------*/

    h->num_rdo_intra_chroma = NUM_INTRA_MODE_CHROMA;

    /* ֡Ԥģʽ */

    h->get_intra_candidates_luma = rdo_get_pred_intra_luma;

    h->get_intra_candidates_chroma = rdo_get_pred_intra_chroma;

    /* AEC */
    h->size_aec_rdo_copy = sizeof(aec_t);
    h->copy_aec_state_rdo = aec_copy_aec_state;

}
